% Procedure MarkovHaploid.m  -Main program
% Non-homogenous markov chain for asexual random reproduction in a diploid 
% population with two different alleles, A and a, in one locus.
% We here considere the haploid GAMETE pool: A and a with size n(1), n(2).
% Ref.: Tyvand, P.A. and Thorvaldsen, S. (2006): Markov model of haploid random
% mating with given distribution of population size. Bulletin of
% Mathematical Biology, 68:807-819.
% Program for Matlab ver 6.x and adjusted to ver 7.x  E-mail: steinar.thorvaldsen@uit.no


% Algorithm:
% Read input parameters (gametes in founder population V0),
% and the number of the distribution function to be used in the function myPDF.m
% For generation t = 1: T
%   Determine row j in transition matrix M 
%   V(t+1)= V(t)*Mj
%   compute statistics in the generation
% end
% Write statistics

clear all;
g=2; %the number of different gametes

%%%%%%%%%%%% INPUT DATA that may be changed: %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generating the initial founder population of each gamete, 
% (remember that the sum must be an even number because the poplation is diploid!):
n(1)=2;
n(2)=6;
% Selecting the number of the probability density function (PDF) to be used in the subsequent generations,
% 1=user defined, 2=user defined, 3=poisson, 4=normal, 5=uniform. See file myPDF.m
%PDFno=[1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2, 2, 1, 1, 1, 2, 2];
PDFno=ones(1,100)  % can alternatively be used when the same PDF is to be used in all generations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp ('Founder vector of gametes  A a :'); n
npop=sum(n)/2;
% The number of different diploid population compositions:
d=npop*(npop+2); % the well known formular:  3+5+7+...+2n+1=n(n+2)
Vinput=zeros(1,d);
i=npop^2 + n(1);
Vinput(i)=1;

disp ('PDF number to be used in the subsequent generations:'); PDFno
disp ('(1=user defined, 2=user defined, 3=poisson, 4=normal, 5=uniform. Edit in file myPDF.m)');
T=length(PDFno); % The number of coming generations

% variables for storing data from each generation, in tabular form (generations vertical):
% index can't be 0 in Matlab, so founder generation has to be No 1
% v=zeros(T+1,d); % can be used to store v, if nessesary.
% v(1,:)=vinput;
Efreq=zeros(T+1,g);
SD=zeros(T+1,g);
Efix=zeros(T+1,2);
Efreq(1,:)=[n(1)/(2*npop), n(2)/(2*npop)];

nmax=npop; %The maximum size of the founder population
% looping through the T generations:
for t=1:T
    disp ('Generation: '); t
    % To find the defined cut-off number, Nmax, in the next generation of the population:
    [f, Nmax] = myPDF(-1, PDFno(t));
    % must compute the distribution in next generation of population, pluss statistics:
    [Vnew, Efreq(t+1,:), SD(t+1,:), Efix(t+1,:)] = VcompGamete(Vinput, nmax, Nmax, PDFno(t));
    % The next population distribution is given by:
    Vinput=Vnew;
    nmax=Nmax;
end

%%%%%%%%%%%% SOME SUGGESTED OUTPUT DATA:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp ('Frequencies of the different gametes:'); Efreq
disp ('Standard Deviation:'); SD
disp ('Probability of fixation in A and in a:'); Efix

plot (SD(:,1))
text(10,SD(10,1), 'SD')
hold on
plot (Efreq(:,1))
text(10,Efreq(10,1), 'Freq. of A')
plot (Efix(:,1))
text(10,Efix(10,1), 'Prob. of fixation in A')
hold off