function [Vnext, E, SD, Efix] = VcompGamete (V, n, Nmax, f);
% A function to compute the next generation's distributions vector, Vnext,
% on the haploid gamete level in different diploid populations of size 1 up to Nmax
% The non-homogenous Markov chain is computed, pluss some statistical parameters.
% By Steinar Thorvaldsen, 2004. steinar@hitos.no  For Matlab ver 6.x (and 7.x)

% V: population distribution vector covering all possible different compositions 
% of haploid gamete populations, up to n members.
% n: number in parent population. NB: n may also be calculated when d=length(V) is
% known, the equation is of 2nd degree, see nargin < 2 below.
% Nmax: cut-off number in offspring population (may be changed in file myPDF.m
% f: selected probability density function (PDF). See file myPDF.m
% E: expected frequences of a gamete
% SD: standard diviation
% Efix: probability of fixation in gamete A or a

% Use: [Vnext, E, SD, Efix] = VcompGamete (V, n, Nmax, f);

g=2; % the total number of gametes in the function VcompGamete
d= length(V); % size of the input population vector

% The function VcompGamete is made flexible conserning use of input parameters:
if nargin < 4
  if nargin < 3
    if nargin < 2
        n= -1 + sqrt(1+d); % can be determined from length of vector
    end
    Nmax=n; % population cut-off stays constant in size
  end
  f=3; % uses poisson distribution as PDF if no parameter given
end

D= Nmax*(Nmax+2); %size of new population distributions vector when g=2 gametes are used
Vnext=zeros(1,D);
E=zeros(1,g); % for accumulated frequencies
SD=zeros(1,g); % for computing standard deviation
Efix=zeros(1,g); % for computing Pr. of fixation
gamete_ratio=zeros(1,g); % for frequense statistics of each gamete

% The dimention of the full transition matrix M is d x D, but we only need to determine the
% j-th column Mj, and then imediately compute its contribution to the next
% generation.
% This method is looped over all j:

for j=1:Nmax*(Nmax+2)
    Mj=zeros(d,1); % we only need one column in the transition matrix M and use this for different rows in M
    % NB: Int* Data Type Functions Round off in Matlab 7 Instead of Truncate as in Matlab 6
    %Nj=double(int32(sqrt(j))) %Matlab ver 6 % number of offspring individuals = integer value of the root of j
    Nj=double(fix(sqrt(j))); %fix round toward zero
    N1=j-Nj^2;
    N2=2*Nj-N1;
    % We also compute the coeffisient in the binomial distribution to be used later:
    binom_coeff = factorial(2*Nj)/(factorial(N1)*factorial(N2)); % = nchoosek(2*Nj,N1);
    trans_coeff=myPDF(Nj,f)*binom_coeff; % the prob. distribution is coded in the function myPDF.m
    for i=1:n*(n+2)
        %ni=double(int32(sqrt(i))); % find integer value in Matlab 6. Must have i < 2.1 billion!
        ni=double(fix(sqrt(i))); % find integer value. Must have i < 2.1 billion!
        n1=i-ni^2;
        n2=2*ni-n1;
        n1_2=n1+n2;
        P1=n1/n1_2;
        P2=n2/n1_2;
        Mj(i,1)=trans_coeff*(P1^(N1)*P2^(N2)); % scaled down Binomial distribution. 
    end        
    Vnext(1,j)= V*Mj; %updating the distribution vector in position j
       
    % To compute the statistics (average frequences of gametes and
    % standard diviation) we need:
    gamete_ratio(1)=N1/(2*Nj); %ratio of gamete 1 represented in position j in the distribution vector
    gamete_ratio(2)=N2/(2*Nj); % ratio of type 2
        
    % To compute the accumulated frequence and variance for each gamete:
    for gamete=1:g
        E(gamete)=E(gamete) + Vnext(1,j)*gamete_ratio(gamete); % Formula E(x)=SUM p(x)*x
        % Preliminary result later used to compute standard diviation (SD):
        SD(gamete)=SD(gamete) + Vnext(1,j)*gamete_ratio(gamete)^2; 
        % Later we use formula Var(x)=p(x)*x^2 - (E(x))^2.
    end
        
    % To compute propability of fixation in gamete A and a:
    if N2==0 & N1>0
        Efix(1)=Efix(1) + Vnext(1,j);
    end
    if N1==0 & N2>0
        Efix(2)=Efix(2) + Vnext(1,j);
    end
end
% sum(Vnext(1,:)) %to test if it is = 1
% To compute the final standard deviation (SD) for each genotype from the variance:
for gamete=1:g
    SD(gamete)=sqrt(SD(gamete) - E(gamete)^2);
end
