function [Vnext, E, SD, Efix] = Vcomp6g (V, n, N);
% A function to compute the next generation's distribution vector for 6 genotypes, 
% Vnext, in an inhomogenous Markov chain, pluss some statistical parameters.

% V: population distribution vector covering all possible different compositions 
% of populations with n members, and a given number (g) of genotypes.
% n: number in parent population. NB: n may also be calculated when d=length(V) is
% known, but in the case g=6 the equation is not easy to solve (5th degree). 
% N: number in offspring population
% E: expected frequences of a genotype
% SD: standard diviation
% Efix: probability of maile fixation in genotype X or x

% Use: [Vnext, E, SD, Efix] = Vcomp6g (V, n, N);

g=6; % the total number of genotypes in the function Vcomp6g
d= length(V); % size of the input population vector

% The function is made flexible conserning input parameters:
if nargin < 3
    N=n; % population stays constant in size
end

D=(factorial(g+N-1))/(factorial(N)*factorial(g-1)); % size of new population vector, valid for g genotypes
Vnext=zeros(1,D);
E=zeros(1,g); % for accumulated frequences
SD=zeros(1,g); % for computing standard diviation
Efix=zeros(1,2); % for computing Pr. of fixation
genotype_ratio=zeros(1,g); % for fequense statistics of each genotype

% The dimention of the full transition matrix M is d x D, and it may be big.
% To save space in computer memory, we only need to determine the
% j-th column Mj, and then imediately compute its contribution to the next
% generation.
% This method is looped over all j.

% But first we vectorise P-values to speed up the algorithm:
P1=zeros(d,1);
P2=zeros(d,1);
P3=zeros(d,1);
for n1=0:n
    for n2=0:n-n1
      for n3=0:n-n1-n2
        for n4=0:n-n1-n2-n3
          for n5=0:n-n1-n2-n3-n4
            n6=n-n1-n2-n3-n4-n5;
            i=icompute([n1 n2 n3 n4 n5 n6]); % compute index in the vector v
            n_male=n1+n2+n3;
            n_female=n4+n5+n6;
            if (n_male) > 0 & (n_female) > 0 % testing the existense of both male and females.
               P1(i)=(((n1+0.5*n2)*(n4+0.5*n5))/(n_male*n_female))*0.5; % also = P4
               P2(i)=(((n1+0.5*n2)*(n6+0.5*n5)+(n3+0.5*n2)*(n4+0.5*n5))/(n_male*n_female))*0.5; % also = P5
               P3(i)=(((n3+0.5*n2)*(n6+0.5*n5))/(n_male*n_female))*0.5; % also = P6
            end
          end % last small n
        end
      end
    end
end % first small n 

%Main loop follows:
for N1=0:N
    for N2=0:N-N1
     for N3=0:N-N1-N2
      for N4=0:N-N1-N2-N3
       for N5=0:N-N1-N2-N3-N4
        N6=N-N1-N2-N3-N4-N5;      
        j=icompute([N1 N2 N3 N4 N5 N6]); % compute index in the vector v in a seperate function
        Mj=zeros(d,1); % we only need one column in the transition matrix M and use this to represent different rows in M
        % We also compute the coeffisient in the multinomial distribution to be used later:
        multinom_coeff=factorial(N)/(factorial(N1)*factorial(N2)*factorial(N3)*factorial(N4)*factorial(N5)*factorial(N6));
        % Transition prob. is given by the multinomial distribution:
        Mj(:,1)=multinom_coeff*(P1.^(N1).*P2.^(N2).*P3.^(N3).*P1.^(N4).*P2.^(N5).*P3.^(N6)); % .^ Array power.
        Vnext(1,j)= V*Mj; %updating the distribution vector in position j
        
        % To compute the statistics (average frequences of genotypes and
        % standard diviation) we need:
        genotype_ratio(1)=N1/N; %ratio of genotype 1 represented in position j in the distribution vector
        genotype_ratio(2)=N2/N; % ratio of type 2
        genotype_ratio(3)=N3/N; % ratio of type 3
        genotype_ratio(4)=N4/N; % ratio of type 4
        genotype_ratio(5)=N5/N; % ratio of type 5
        genotype_ratio(6)=N6/N; % ratio of type 6
        
        % To compute the accumulated frequence and variance for each genotype:
        for genotype=1:g
            E(genotype)=E(genotype) + Vnext(1,j)*genotype_ratio(genotype); % Formula E(x)=SUM p(x)*x
            % Preliminary result later used to compute standard diviation (SD):
            SD(genotype)=SD(genotype) + Vnext(1,j)*genotype_ratio(genotype)^2; 
            % Later we use formula Var(x)=p(x)*x^2 - (E(x))^2.
        end
        
        % To compute propability of fixation in cromosomes A and a:
        if (N2==0 & N3==0 & N5==0 & N6==0)
            Efix(1)=Efix(1) + Vnext(1,j)*(genotype_ratio(1)+genotype_ratio(4));
        end
        if (N1==0 & N2==0 & N4==0 & N5==0)
            Efix(2)=Efix(2) + Vnext(1,j)*(genotype_ratio(3)+genotype_ratio(6));
        end
       end % last big N in loop
       end
    end
   end
end % first big N in loop
% SD
% To compute the final standard diviation (SD) for each genotype from the variance:
for genotype=1:g
    SD(genotype)=sqrt(SD(genotype) - E(genotype)^2);
end

% For testing output:
% V
% Vnext
% sum(Vnext)






