% Procedure MoranHapMicroEv.m
% Moran model with homogenous markov chain for asexual random reproduction 
% in a diploid population with two different alleles, A and a, in one locus.
% We here considere the GAMETE pool: A and a with size n(1), n(2).
% Mutation rates (delta, epsi), and selection rates (beta) are given.
% Ref.: Tyvand, P.A. and Thorvaldsen, S. (2010): Exact Markov chains versus 
% diffusion theory for haploid random mating. Mathematical Biosciences, 225, 18-23.

% Algorithm:
% Read input parameters (gametes in founder population V0), gamate size, 
% mutation rate, and selection rates in the coming generations.
% Remark: One Wrigt-Fisher generation equals npop Moran steps, where npop=diploid size.
% For WFgeneration t = 1: T
%   Determine row j in transition matrix M 
%   V(t+1)= V(t)*Mj
%   compute statistics in npop Moran-generations
% end
% Write statistics

clear all;
g=2; %the number of different gametes

%%%%%%%%%%%% INPUT DATA that may be changed: %%%%%%%%%%%%%%%%%%%%%%%%%%
% Generating the initial founder population of each gamete, 
% (remember that the sum must be an even number):
n(1)=10; %gamete A
n(2)=10; %gamete a
% Giving the number of diploid individuals in the subsequent generations:
npop=sum(n)/2;

T=20; % The number of coming Wright-Fisher generations
% Giving the mutation rate from A to a, and a to A, in the subsequent generations:
%delta1=[0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01];
%epsi1=[0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01];
delta1=0*ones(1,T);  % can alternatively be used for constant mutation rate A to a
epsi1=0*ones(1,T);  % can alternatively be used for constant mutation rate a to A
% Giving the second level mutation rate from A to a, and a to A, in the subsequent generations:
%delta2=[0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01];
%epsi2=[0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01 0.01];
delta2=0.04*ones(1,T);  % can alternatively be used for constant mutation rate A to a
epsi2=0.02*ones(1,T);  % can alternatively be used for constant mutation rate a to A
% Giving the selection rate for alleles a and A in the subsequent generations:
%s1=[2 2 2 2 2 2 2 2 2 2 2 2 2 2 2];
%s2=[2 2 2 2 2 2 2 2 2 2 2 2 2 2 2];
% can alternatively be used for constant selection rate
s1=0*ones(1,T);  % constant fitness rate for A. Must not be =1
s2=-0.2*ones(1,T);  % constant fitness rate for a. Must not be =1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

format short;
disp ('Discrete Moran model of haploid micro-evolution');
disp (['Founder population vector of gametes A and a: ', num2str(n)])
% The number of different diploid population compositions:
d=2*npop+1; % the number of different gamete populations
Vinput=zeros(1,d);
i=n(1)+1;
Vinput(i)=1;
disp (['Constant diploid population size in the subsequent generations:  ', num2str(npop)])
disp (['Fitness rate for A in the subsequent generations:  ', num2str(s1)])
disp (['Fitness rate for a in the subsequent generations:  ', num2str(s2)]);
disp (['Mutation rate from allele A to a in the subsequent generations:  ', num2str(delta1)]); 
disp (['Mutation rate from allele a to A in the subsequent generations:  ', num2str(epsi1)]); 
disp (['Second mutation rate from allele A to a :  ', num2str(delta2)]); 
disp (['Second mutation rate from allele a to A :  ', num2str(epsi2)]); 

% variables for storing data from each generation, in tabular form (generations vertical):
% index can't be 0 in Matlab, so founder generation has to be No 1
% v=zeros(T+1,d); % can be used to store v, if nessesary.
% v(1,:)=vinput;
v=zeros(T,d);
Efreq=zeros(T+1,g);
SD=zeros(T+1,g);
Efix=zeros(T+1,2);
Efreq(1,:)=[n(1)/(2*npop), n(2)/(2*npop)];
%v{1}=Vinput; %use cell array to store data of unequal size
% looping through the T generations:
for t=1:T
    disp (['Starting Wright-Fisher generation: ', num2str(t),'... '])
    for tmor=1:npop
      % must determine the distribution in next population, pluss statistics:
      [Vnew, Efreq(t+1,:), SD(t+1,:), Efix(t+1,:)] = VcompMoranHaploid(Vinput, npop, delta1(t),epsi1(t),delta2(t),epsi2(t),s1(t),s2(t));
      % The next population distribution is given by:
      Vinput=Vnew;
    end
    v(t,:)=Vnew; %use array to store data when each new Wright-Fisher generation is passed
    format long;
end
%vc=num2cell(v)
%Vplot2 (vc, 0.5); %plotting distribution vector over its generations, mask resolution 0.5
%format long;
sum(v(T,:))
%disp ('Frequencies of the different gametes:'); Efreq
%disp ('Standard Deviation:'); SD
%disp ('Probability of fixation in A and in a:'); Efix

    


