function [Vnext, E, SD, Efix] = VcompMoranHaploid (V, n, delta1, epsi1, delta2, epsi2, s1, s2);
% A function to compute the next generation's distribution vector for 2 genotypes, Vnext, 
% in a homogenous Markov chain of a Moran model, pluss some statistical parameters.

% V: population distribution vector covering all possible different compositions 
% of gamete populations, up to n members.
% n: number of individuals in parent and offspring population (NB! n=N)
% NB: n may also be calculated when d=length(V) is known, see below.
% delta1: the mutation rate from allele A to a 
% epsi1: the mutation rate from allele a to A 
% delta2: the mutation rate from allele A to a 
% epsi2: the mutation rate from allele a to A 
% s1: the selection rate for reproduction: >0 in faviour of A, <0 in favoiour of a
% s2: the selection rate for survival: >0 in faviour of A, <0 in favoiour of a
% E: expected frequences of a gamete
% SD: standard diviation
% Efix: probability of fixation in gamete A or a

% Use: [Vnext, E, SD, Efix] = VcompMoranHaploid (V, n, delta1, epsi1, delta2, epsi2, s1, s2);

g=2; % the total number of gametes in the function VcompMicroev2g
d= length(V); % size of the input and output population vector

% The function is made flexible conserning input parameters:
if nargin < 8
 if nargin < 7
  if nargin < 6
   if nargin < 5
    if nargin < 4
        if nargin < 3
            if nargin < 2
              n = (d-1)/2; % can be determined from length of vector
            end
        delta1=0;
        end
    epsi1=0;
    end
   delta2=0;
   end
  epsi2=0;
  end
 s1=0;
 end
s2=0;
end

Vnext=zeros(1,d);
E=zeros(1,g); % for accumulated frequencies
SD=zeros(1,g); % for computing standard deviation
Efix=zeros(1,g); % for computing Pr. of fixation
gamete_ratio=zeros(1,g); % for frequense statistics of each gamete

% The dimention of the full transition matrix M is d x d, and it may be big.
% To save space in computer memory, we only need to determine the
% j-th column Mj, and then imediately compute its contribution to the next
% generation.
% This method is looped over all j:

% But first we vectorise P1 and P2-values to speed up the algorithm:
P1=zeros(d,1); %probability offspring A
P2=zeros(d,1); %probability offspring a
Q1=zeros(d,1); %probability removal A
Q2=zeros(d,1); %probability removal a
wtemp1=1-s1;
wtemp2=1-s2;
for i=1:d  % remember d=2*n+1;
    n1=i-1;
    n2=d-i;
    % One gamete is copied befor one gamete is removed (not oposite):
    temp3=n1+n2-s1*n2-delta1*s1*n1+epsi1*s1*n2;
    P1(i)=(n1-delta1*n1+epsi1*n2-delta2*n1+delta1*delta2*n1-delta2*epsi1*n2 + wtemp1*(delta1*epsi2*n1+epsi2*n2-epsi1*epsi2*n2))/temp3;
    P2(i)=(n1*delta2-delta1*delta2*n1+delta2*epsi1*n2 + wtemp1*(delta1*n1+n2-epsi1*n2-delta1*epsi2*n1-epsi2*n2+epsi1*epsi2*n2))/temp3;
    temp4=n1+wtemp2*n2;
    Q1(i)=n1/temp4;
    Q2(i)=(wtemp2*n2)/temp4;
end    

% The first border j=1:
Mi=zeros(d,1); % we only need one column in the transition matrix M and use this for different rows
Mi(1,1)=Q1(1)*P1(1)+Q2(1)*P2(1);
Mi(2,1)=Q1(2)*P2(2);%Mj(2,1)=Q2(2)*P1(2);
Vnext(1,1)= V*Mi; %updating the distribution vector in position 1

% The innbetween:
for j=2:d-1
    Mi=zeros(d,1); % we only need one column in the transition matrix M
    Mi(j-1,1)=Q2(j-1)*P1(j-1);%Mj(j-1,1)=Q1(j-1)*P2(j-1);
    Mi(j,1)=Q1(j)*P1(j)+Q2(j)*P2(j);
    Mi(j+1,1)=Q1(j+1)*P2(j+1);%Mj(j+1,1)=Q2(j+1)*P1(j+1);
Vnext(1,j)= V*Mi; %updating the distribution vector in position j
end

% The last j=d
Mi=zeros(d,1); % we only need one column in the transition matrix M
Mi(d-1,1)=Q2(d-1)*P1(d-1);%Mj(d-1,1)=Q1(d-1)*P2(d-1);
Mi(d,1)=Q1(d)*P1(d)+Q2(d)*P2(d);           
Vnext(1,d)= V*Mi; %updating the distribution vector in its last position d
%testP=sum(P1,1) + sum(P2,1)

% To compute the statistics (average frequences of gametes and
% standard diviation) we need:
%    gamete_ratio(1)=N1/(2*N); %ratio of gamete 1 represented in position j in the distribution vector
 %   gamete_ratio(2)=N2/(2*N); % ratio of type 2
        
    % To compute the accumulated frequence and variance for each gamete:
  %  for gamete=1:g
  %      E(gamete)=E(gamete) + Vnext(1,j)*gamete_ratio(gamete); % Formula E(x)=SUM p(x)*x
        % Preliminary result later used to compute standard diviation (SD):
  %      SD(gamete)=SD(gamete) + Vnext(1,j)*gamete_ratio(gamete)^2; 
        % Later we use formula Var(x)=p(x)*x^2 - (E(x))^2.
  %  end
        
    % To compute propability of fixation in gamete A and a:
  %  if N2==0 & N1>0
  %      Efix(1)=Efix(1) + Vnext(1,j);
  %  end
  %  if N1==0 & N2>0
  %      Efix(2)=Efix(2) + Vnext(1,j);
  %  end
%end

% To compute the final standard deviation (SD) for each genotype from the variance:
%for gamete=1:g
%    SD(gamete)=sqrt(SD(gamete) - E(gamete)^2);
%end

% For testing output:
% sum(Vnext(1,:)) %to test if it is = 1
% V
% Vnext
