function [Vnext, E, SD, Efix] = Vcomp2dip (V, n, N, s);
% A function to compute the next generation's distribution vector, Vnext,
% in a diploid population with a given probability, s, of self mating.
% The inhomogenous Markov chain is computed, pluss some statistical parameters.

% V: population distribution vector covering all possible different compositions 
% of populations, with n members and a given number (g) of genotypes.
% n: number in parent population. NB: n may also be calculated when d=length(V) is
% known, and in the case g=3 the equation is easy to solve, and n comes out as 
% the positive root of a 2.deg. equation, see below.
% N: number in offspring population
% s: the probability of self mating. Pure self-mating: s=1.
% E: expected frequences of a genotype
% SD: standard diviation
% Efix: probability of fixation in genotype AA or aa

% Use: [Vnext, E, SD, Efix] = Vcomp2dip (V, n, N, s);

g=3; % the total number of genotypes in the function Vcomp2dip
d= length(V); % size of the input population vector

% The function Vcomp2dip is made flexible conserning use of input parameters:
if nargin < 4
  if nargin < 3
    if nargin < 2
        n= 0.5*(-3 + sqrt(1+8*d));
    end
    N=n; % population stays constant in size
  end
  s=1/n; % the probability of self-mating is equal to probability of cross-mating
end

D= (N+2)*(N+1)/2; %size of new population vector when g=3 genotypes are used
Vnext=zeros(1,D);
E=zeros(1,g); % for accumulated frequencies
SD=zeros(1,g); % for computing standard deviation
Efix=zeros(1,2); % for computing Pr. of fixation
genotype_ratio=zeros(1,g); % for fequense statistics of each genotype

% The dimention of the full transition matrix M is d x D, and it may be big.
% To save space in computer memory, we only need to determine the
% j-th column Mj, and then imediately compute its contribution to the next
% generation.
% This method is looped over all j:

for N1=0:N
    for N2=0:N-N1
        N3=N-N1-N2;      
        j=1+N1+(N1+N2)*(N1+N2+1)*0.5;
        Mj=zeros(d,1); % we only need one column in the transition matrix M and use this for different rows in M
        % We also compute the coeffisient in the trinomial distribution to be used later:
         multinom_coeff=factorial(N)/(factorial(N1)*factorial(N2)*factorial(N3));
           for n1=0:n
               for n2=0:n-n1
                   n3=n-n1-n2;
                   i=1+n1+(n1+n2)*(n1+n2+1)*0.5;
                   p=n1/n;
                   q=n2/n;
                   r=n3/n;
                   P=(n*(1-s)*(p+0.5*q)^2 + (s*n-1)*(p+0.25*q))/(n-1);
                   Q=(n*(1-s)*(p*q+r*q+2*p*r+0.5*(q^2)) + (s*n-1)*q*0.5)/(n-1);
                   R=(n*(1-s)*(r+0.5*q)^2 + (s*n-1)*(r+0.25*q))/(n-1);
                   Mj(i,1)=multinom_coeff*(P^(N1)*Q^(N2)*R^(N3)); % Trinomial distribution
               end
           end        
        Vnext(1,j)= V*Mj; %updating the distribution vector in position j
        
        % To compute the statistics (average frequences of genotypes and
        % standard diviation) we need:
        genotype_ratio(1)=N1/N; %ratio of genotype 1 represented in position j in the distribution vector
        genotype_ratio(2)=N2/N; % ratio of type 2
        genotype_ratio(3)=N3/N; % ratio of type 3
        
        % To compute the accumulated frequence and variance for each genotype:
        for genotype=1:g
            E(genotype)=E(genotype) + Vnext(1,j)*genotype_ratio(genotype); % Formula E(x)=SUM p(x)*x
            % Preliminary result later used to compute standard diviation (SD):
            SD(genotype)=SD(genotype) + Vnext(1,j)*genotype_ratio(genotype)^2; 
            % Later we use formula Var(x)=p(x)*x^2 - (E(x))^2.
        end
        
        % To compute propability of fixation in gene A and a:
        if N2==0 & N3==0 & N1>0
            Efix(1)=Efix(1) + Vnext(1,j);
        end
        if N1==0 & N2==0 & N3>0
            Efix(2)=Efix(2) + Vnext(1,j);
        end
    end
end
% SD
% To compute the final standard deviation (SD) for each genotype from the variance:
for genotype=1:g
    SD(genotype)=sqrt(SD(genotype) - E(genotype)^2);
end







