function [Vnext, E, SD, Efix] = VcompWrightFisherHaploid (V, n, N, delta, epsi, w);
% A function to compute the next generation's distribution vector for 2 genotypes, Vnext, 
% in an inhomogenous Markov chain, pluss some statistical parameters.

% V: population distribution vector covering all possible different compositions 
% of gamete populations, up to n members.
% n: number of individuals in parent population. NB: n may also be calculated 
% when d=length(V) is known, see below.
% N: number of individuals in offspring population
% delta: the mutation rate from allele A to a 
% epsi: the mutation rate from allele a to A 
% w: the selection rate for alleles a and A, respectively
% E: expected frequences of a gamete
% SD: standard diviation
% Efix: probability of fixation in gamete A or a

% Use: [Vnext, E, SD, Efix] = VcompWrightFisherHaploid (V, n, N, delta, epsi, w);

g=2; % the total number of gametes in the function VcompMicroev2g
d= length(V); % size of the input population vector
% The function is made flexible conserning input parameters:
if nargin < 5
    if nargin < 4
        if nargin < 3
            if nargin < 2
                n = (d-1)/2; % can be determined from length of vector
            end
        N=n; % population stays constant in size
        end
    epsi=0;
    end
beta=[1 1];   
end

D=2*N+1; %the new number of different gamete populations
Vnext=zeros(1,D);
E=zeros(1,g); % for accumulated frequencies
SD=zeros(1,g); % for computing standard deviation
Efix=zeros(1,g); % for computing Pr. of fixation
gamete_ratio=zeros(1,g); % for frequense statistics of each gamete

% The dimention of the full transition matrix M is d x D, and it may be big.
% To save space in computer memory, we only need to determine the
% j-th column Mj, and then imediately compute its contribution to the next
% generation.
% This method is looped over all j:

% But first we vectorise P1 and P2-values to speed up the algorithm:
P1=zeros(d,1);
P2=zeros(d,1);
for i=1:d  % remenber d=2*n+1;
    n1=i-1;
    n2=d-i;
    %temp=w(1)*((1-delta)*n1+epsi*n2)+w(2)*(delta*n1+(1-epsi)*n2);
    %P1(i)=(w(1)*((1-delta)*n1+epsi*n2))/temp;
    %P2(i)=(w(2)*(delta*n1+(1-epsi)*n2))/temp;
    % when the mutation takes place after the individ is formed, but before it breeeds:
    temp=w(1)*n1+w(2)*n2;
    P1(i)=((1-delta)*w(1)*n1+epsi*w(2)*n2)/temp;
    P2(i)=(delta*w(1)*n1+(1-epsi)*w(2)*n2)/temp;
end    

for j=1:D
    Mj=zeros(d,1); % we only need one column in the transition matrix M and use this for different rows in M
    N1=j-1;
    N2=D-j;
    % We compute the coeffisient in the binomial distribution:
    binom_coeff=nchoosek(N1+N2,N1); % =factorial(N1+N2)/(factorial(N1)*factorial(N2));
    Mj(:,1)=binom_coeff*(P1.^(N1).*P2.^(N2)); % Binomial distribution (.^ Array power) 
    Vnext(1,j)= V*Mj; %updating the distribution vector in position j
    
    % To compute the statistics (average frequences of gametes and
    % standard diviation) we need:
    gamete_ratio(1)=N1/(2*N); %ratio of gamete 1 represented in position j in the distribution vector
    gamete_ratio(2)=N2/(2*N); % ratio of type 2
        
    % To compute the accumulated frequence and variance for each gamete:
    for gamete=1:g
        E(gamete)=E(gamete) + Vnext(1,j)*gamete_ratio(gamete); % Formula E(x)=SUM p(x)*x
        % Preliminary result later used to compute standard diviation (SD):
        SD(gamete)=SD(gamete) + Vnext(1,j)*gamete_ratio(gamete)^2; 
        % Later we use formula Var(x)=p(x)*x^2 - (E(x))^2.
    end
        
    % To compute propability of fixation in gamete A and a:
    if N2==0 & N1>0
        Efix(1)=Efix(1) + Vnext(1,j);
    end
    if N1==0 & N2>0
        Efix(2)=Efix(2) + Vnext(1,j);
    end
end

% To compute the final standard deviation (SD) for each genotype from the variance:
for gamete=1:g
    SD(gamete)=sqrt(SD(gamete) - E(gamete)^2);
end

% For testing output:
% sum(Vnext(1,:)) %to test if it is = 1
% V
% Vnext
