function [Vnext, E, SD, Efix] = VcompMF6g (V, m, n, M, N, d1, e1, d2, e2, w1, w2, w3, w4, w5, w6)
% A function to compute the next generation's distribution matrix for 3+3 genotypes, 
% Vnext, in an inhomogenous Markov chain, pluss some statistical parameters.

% V: population distribution matrix covering all possible different compositions 
% of diploid populations with m males and n females.
% m: number of males in parent population. 
% n: number of females in parent population.
% NB: m and n may also be calculated when V ik known, m=length(V)... 
% M: number of males in offspring population
% N: number of females in offspring population
% d1 (delta1): the mutation rate from allele A to a 
% e1 (epsi1): the mutation rate from allele a to A 
% d2 (delta2): the mutation rate from allele Ao to ao (females)
% e2 (epsi2): the mutation rate from allele ao to Ao (females)
% w1,w2,w3,w4,w5,w6: fitness parameter for genotype AA,Aa,aa,AoAo,Aoao,aoao

% E: expected frequences of a genotype
% SD: standard diviation
% Efix: probability of maile fixation in genotype X or x

% Use: [Vnext, E, SD, Efix] = VcompMF6g (V, m, n, M, N);

g=3; % the number of different genotypes
gg=g+g; % the total number of genotypes (males + females)
[dm,dn] = size(V); % size of the input population matrix

% The function is made flexible conserning input parameters:
if nargin < 5
    N=n; % female population stays constant in size
    if nargin <4
      M=m; % males stays constant in size
    end
end

% Variables:
DM=(M+1)*(M+2)/2;
DN=(N+1)*(N+2)/2;
Mijkl=zeros(dm,dn,DM,DN); % % the full transition matrix Mijkl is 4-dimentional
Vnext=zeros(DM,DN);
E=zeros(1,gg); % for accumulated frequences
SD=zeros(1,gg); % for computing standard diviation
Efix=zeros(1,2); % for computing Pr. of fixation
genotype_ratio=zeros(DM,DN,gg); % for fequense statistics of each genotype

if m > 0 & n > 0 % testing the existense of both male and females.
  % First we precomputes P- and Q-values to speed up the algorithm:
  P1=zeros(dm,dn);
  P2=zeros(dm,dn);
  P3=zeros(dm,dn);
  Q1=zeros(dm,dn);
  Q2=zeros(dm,dn);
  Q3=zeros(dm,dn);
  for m1=0:m
    for m2=0:m-m1
      m3=m-m1-m2;
      i=icompute([m1 m2 m3]); % compute index in the matrix V
      for n1=0:n
        for n2=0:n-n1
          n3=n-n1-n2;
          j=icompute([n1 n2 n3]); % compute index in the matrix V
          mn4=m*n*4; %temporary factors first:
          f1=(1-d1)*(2*m1+m2) + e1*(m2+2*m3);
          f2=(1-d2)*(2*n1+n2) + e2*(n2+2*n3);
          f3=d2*(2*n1+n2) + (1-e2)*(n2+2*n3);
          f4=d1*(2*m1+m2) + (1-e1)*(m2+2*m3);
          P1t=f1*f2/mn4; % also = Q1t
          P2t=((f1*f3)+(f4*f2))/mn4; % also = Q2t
          P3t=f4*f3/mn4; % also = Q3t
          Pdenom=w1*P1t+w2*P2t+w3*P3t;
          Qdenom=w4*P1t+w5*P2t+w6*P3t; %because Qt=Pt
          P1(i,j)=w1*P1t/Pdenom;
          P2(i,j)=w2*P2t/Pdenom;
          P3(i,j)=w3*P3t/Pdenom;
          Q1(i,j)=w4*P1t/Qdenom; % Q1t=P1t
          Q2(i,j)=w5*P2t/Qdenom; % Q2t=P2t
          Q3(i,j)=w6*P3t/Qdenom; % Q3t=P3t
        end % last small n
      end
    end
  end % first small m 

  
  %Main loop follows:
  for M1=0:M
    for M2=0:M-M1
      M3=M-M1-M2;
      k=icompute([M1 M2 M3]); % compute first index in the matrix Vnext in a seperate function
      coeffM=factorial(M)/(factorial(M1)*factorial(M2)*factorial(M3)); % first factor in the distribution
      for N1=0:N
       for N2=0:N-N1
        N3=N-N1-N2;      
        l=icompute([N1 N2 N3]); % compute second index in the matrix Vnext in a seperate function
        % Coeffisient in the distribution:
        coeff=coeffM*(factorial(N)/(factorial(N1)*factorial(N2)*factorial(N3)));
        for m1=0:m
            for m2=0:m-m1
                m3=m-m1-m2;
                i=icompute([m1 m2 m3]); % compute index in the matrix v
                for n1=0:n
                    for n2=0:n-n1
                        n3=n-n1-n2;
                        j=icompute([n1 n2 n3]); % compute index in the matrix v
                        Mijkl(i,j,k,l)=coeff*(P1(i,j)^M1*P2(i,j)^M2*P3(i,j)^M3)*(Q1(i,j)^N1*Q2(i,j)^N2*Q3(i,j)^N3); 
                    end % last small n
                end
            end
        end % first small m 

        % To compute the statistics we need to store the ratio of genotypes 
        % represented in position k,l in the distribution matrix:
        genotype_ratio(k,l,1)=M1/M; %ratio of male genotype 1
        genotype_ratio(k,l,2)=M2/M; % ratio of male type 2
        genotype_ratio(k,l,3)=M3/M; % ratio of male type 3
        genotype_ratio(k,l,4)=N1/N; % ratio of female genotype 1
        genotype_ratio(k,l,5)=N2/N; % ratio of female type 2
        genotype_ratio(k,l,6)=N3/N; % ratio of female type 3
     
       end % last big N in loop
      end
    end
  end % first big M in loop
end % if test for existence of both males & females


% Uppdating the distribution matrix:
for i=1:dm
    for j=1:dn
        Vnext(:,:)= Vnext(:,:)+V(i,j)*squeeze(Mijkl(i,j,:,:)); %squeeze removes singleton dimensions
    end
end

% To compute the frequence (E) and standard diviation (SD) for male and fermale genotypes:
%for k=1:DM
%    for l=1:DN
%     for genotype=1:g %males
%      E(genotype)=E(genotype)+Vnext(k,l)*genotype_ratio(k,l,genotype);% Formula: E(x)=SUM p(x)*x
%      % Preliminary result later used to compute standard diviation (SD):
%      SD(genotype)=SD(genotype) + Vnext(k,l)*(genotype_ratio(k,l,genotype)^2); 
%      % Later we use formula Var(x)=p(x)*x^2 - (E(x))^2.
%     end
%     for genotype=g+1:gg %females
%      E(genotype)=E(genotype)+Vnext(k,l)*genotype_ratio(k,l,genotype);% Formula: E(x)=SUM p(x)*x
%      % Preliminary result later used to compute standard diviation (SD):
%      SD(genotype)=SD(genotype) + Vnext(k,l)*(genotype_ratio(k,l,genotype)^2); 
%      % Later we use formula Var(x)=p(x)*x^2 - (E(x))^2.
%     end
%   end
%end
%% To compute the final standard diviation (SD) for each genotype from the variance:
%for genotype=1:gg
%    SD(genotype)=sqrt(SD(genotype) - E(genotype)^2);
%end

% To compute the frequence (E) and standard diviation (SD) for male and fermale genotypes
% (vectorised version):
for genotype=1:g %males
    E(genotype)=sum(sum(Vnext(:,:).*genotype_ratio(:,:,genotype)));% Formula: E(x)=SUM p(x)*x
    Var=sum(sum(Vnext(:,:).*(genotype_ratio(:,:,genotype)).^2)) - E(genotype)^2;
    SD(genotype)=sqrt(Var);           % Formula: Var(x)=SUM p(x)*x^2 - (E(x))^2
end
for genotype=g+1:gg %females
    E(genotype)=sum(sum(Vnext(:,:).*(genotype_ratio(:,:,genotype))));% Formula: E(x)=SUM p(x)*x
    Var=sum(sum(Vnext(:,:).*(genotype_ratio(:,:,genotype)).^2)) - E(genotype)^2;
    SD(genotype)=sqrt(Var);           % Formula: Var(x)=SUM p(x)*x^2 - (E(x))^2
end


% To compute propability of gamete fixation in both sexes:
% In gamete A:
[fixIndex_k,fixIndex_l] = find(genotype_ratio(:,:,1)>=1-eps & genotype_ratio(:,:,4)>=1-eps);
Efix(1)=sum(sum(Vnext(fixIndex_k,fixIndex_l)));
% In gamete a:
[fixIndex_k,fixIndex_l] = find(genotype_ratio(:,:,3)>=1-eps & genotype_ratio(:,:,6)>=1-eps);
Efix(2)=sum(sum(Vnext(fixIndex_k,fixIndex_l)));

% For testing output:
% V
%Vnext
%genotype_ratio(:,:,1)
%genotype_ratio(:,:,4)

% sum(Vnext)
