function [Vnext, E, SD, Efix] = VcompWF5g (V, m, n, M, N, d1, e1, d2, e2, w1, w2, w3, w4, w5, dd1, ee1, dd2, ee2)
% A function to compute the next generation's distribution matrix for 2+3 genotypes, 
% Vnext, in an inhomogenous Markov chain, pluss some statistical parameters.

% V: population distribution matrix covering all possible different compositions 
% of diploid populations with m males and n females.
% m: number of males in parent population. 
% n: number of females in parent population.
% NB: m and n may also be calculated when V ik known, m=length(V)... 
% M: number of males in offspring population
% N: number of females in offspring population
% d1 (delta1): the mutation rate from male gamete A to a 
% e1 (epsi1): the mutation rate from male gamete a to A 
% d2 (delta2): the mutation rate from female gamete Ao to ao
% e2 (epsi2): the mutation rate from female gamete ao to Ao (females)
% w1,w2,w3,w4,w5: fitness parameter for selection of genotype A,a,AoAo,Aoao,aoao respectively
% dd1, ee1, dd2, ee2: the late mutation rate

% E: expected frequences of a genotype
% SD: standard diviation
% Efix: probability of fixation in both sexes in genotype A or a

% Use: [Vnext, E, SD, Efix] = VcompMF5g (V, m, n, M, N...);

gg=5; % the total number of genotypes (males + females)
[dm,dn] = size(V); % size of the input population matrix

% Variables:
DM=M+1;
DN=(N+1)*(N+2)/2;
Mijkl=zeros(dm,dn,DM,DN); % % the full transition matrix Mijkl is 4-dimentional
Vnext=zeros(DM,DN);
E=zeros(1,gg); % for accumulated frequences
SD=zeros(1,gg); % for computing standard diviation
Efix=zeros(1,2); % for computing Pr. of fixation
genotype_ratio=zeros(DM,DN,gg); % for fequense statistics of each genotype

if m > 0 & n > 0 % testing the existense of both male and females.
  % First we precomputes P- and Q-values to speed up the algorithm:
  P1=zeros(dm,dn);
  P2=zeros(dm,dn);
  Q1=zeros(dm,dn);
  Q2=zeros(dm,dn);
  Q3=zeros(dm,dn);
  for m1=0:m
      m2=m-m1;
      i=icompute([m1 m2]); % compute index in the matrix V
      for n1=0:n
        for n2=0:n-n1
          n3=n-n1-n2;
          j=icompute([n1 n2 n3]); % compute index in the matrix V
          f1=m1/m; %some temporary factors first
          f2=m2/m;
          dnom2=2*n;
          %Probabilities with no mutations and selections:
          P01=(2*n1+n2)/dnom2;
          P02=(2*n3+n2)/dnom2;
          Q01=f1*P01;
          Q03=f2*P02;
          Q02=f1*P02+f2*P01;
          f3=1-d2;
          f4=1-e2;
          %early mutations and selection:
          P1t=(1-d1)*P01 + e1*P02;
          P2t=d1*P01 + (1-e1)*P02;
          Q1t=f3*f3*Q01 + f3*e2*Q02 + e2*e2*Q03;
          Q2t=2*f3*d2*Q01 + (f3-e2+2*d2*e2)*Q02 + 2*e2*f4*Q03;
          Q3t=d2*d2*Q01 + d2*f4*Q02 + f4*f4*Q03;
          P1tt=w1*P1t;
          P2tt=w2*P2t;
          Q1tt=w3*Q1t;
          Q2tt=w4*Q2t;
          Q3tt=w5*Q3t;
          %eventual late mutations:
          f3=1-dd2; %some more temporary factors
          f4=1-ee2;
          P1ttt=(1-dd1)*P1tt + ee1*P2tt;
          P2ttt=dd1*P1tt + (1-ee1)*P2tt;
          Q1ttt=f3*f3*Q1tt + f3*ee2*Q2tt + ee2*ee2*Q3tt;
          Q2ttt=2*f3*dd2*Q1tt + (f3-ee2+2*dd2*ee2)*Q2tt + 2*ee2*f4*Q3tt;
          Q3ttt=dd2*dd2*Q1tt + dd2*f4*Q2tt + f4*f4*Q3tt;
          Pdenom=P1ttt+P2ttt;
          Qdenom=Q1ttt+Q2ttt+Q3ttt;
          P1(i,j)=P1ttt/Pdenom;
          P2(i,j)=P2ttt/Pdenom;
          Q1(i,j)=Q1ttt/Qdenom;
          Q2(i,j)=Q2ttt/Qdenom;
          Q3(i,j)=Q3ttt/Qdenom;
        end % last small n
      end
  end % first small m 

  
  %Main loop follows:
  for M1=0:M
      M2=M-M1;
      k=icompute([M1 M2]); % compute first index in the matrix Vnext in a seperate function
      coeffM=factorial(M)/(factorial(M1)*factorial(M2)); % first factor in the distribution
      for N1=0:N
       for N2=0:N-N1
        N3=N-N1-N2;      
        l=icompute([N1 N2 N3]); % compute second index in the matrix Vnext in a seperate function
        % Coeffisient in the distribution:
        coeff=coeffM*(factorial(N)/(factorial(N1)*factorial(N2)*factorial(N3)));
        for m1=0:m
            m2=m-m1;
            i=icompute([m1 m2]); % compute index in the matrix v
            for n1=0:n
                for n2=0:n-n1
                    n3=n-n1-n2;
                    j=icompute([n1 n2 n3]); % compute index in the matrix v
                    Mijkl(i,j,k,l)=coeff*(P1(i,j)^M1*P2(i,j)^M2*Q1(i,j)^N1*Q2(i,j)^N2*Q3(i,j)^N3); 
                end % last small n
            end
        end % first small m 

        % To compute the statistics we need to store the ratio of genotypes 
        % represented in position k,l in the distribution matrix:
        genotype_ratio(k,l,1)=M1/M; %ratio of male genotype 1
        genotype_ratio(k,l,2)=M2/M; % ratio of male type 2
        genotype_ratio(k,l,3)=N1/N; % ratio of female genotype 1
        genotype_ratio(k,l,4)=N2/N; % ratio of female type 2
        genotype_ratio(k,l,5)=N3/N; % ratio of female type 3
     
       end % last big N in loop
      end
  end % first big M in loop
end % if test for existence of both males & females


% Uppdating the distribution matrix:
for i=1:dm
    for j=1:dn
        Vnext(:,:)= Vnext(:,:)+V(i,j)*squeeze(Mijkl(i,j,:,:)); %squeeze removes singleton dimensions
    end
end


% To compute the frequence (E) and standard diviation (SD) for male and fermale genotypes
% (vectorised version):
for genotype=1:2 %males
    E(genotype)=sum(sum(Vnext(:,:).*genotype_ratio(:,:,genotype)));% Formula: E(x)=SUM p(x)*x
    Var=sum(sum(Vnext(:,:).*(genotype_ratio(:,:,genotype)).^2)) - E(genotype)^2;
    SD(genotype)=sqrt(Var);           % Formula: Var(x)=SUM p(x)*x^2 - (E(x))^2
end
for genotype=3:5 %females
    E(genotype)=sum(sum(Vnext(:,:).*(genotype_ratio(:,:,genotype))));% Formula: E(x)=SUM p(x)*x
    Var=sum(sum(Vnext(:,:).*(genotype_ratio(:,:,genotype)).^2)) - E(genotype)^2;
    SD(genotype)=sqrt(Var);           % Formula: Var(x)=SUM p(x)*x^2 - (E(x))^2
end


% To compute propability of gamete fixation in both sexes:
% In gamete A:
[fixIndex_k,fixIndex_l] = find(genotype_ratio(:,:,1)>=1-eps & genotype_ratio(:,:,3)>=1-eps);
Efix(1)=sum(sum(Vnext(fixIndex_k,fixIndex_l)));
% In gamete a:
[fixIndex_k,fixIndex_l] = find(genotype_ratio(:,:,2)>=1-eps & genotype_ratio(:,:,5)>=1-eps);
Efix(2)=sum(sum(Vnext(fixIndex_k,fixIndex_l)));

% For testing output:
% V
% Vnext
% sum(Vnext)
